@@ -33,6 +33,10 @@
 #include "openmodelica.h"
 #include "omc_config.h"
 
+extern const char* System_modelicaPlatform();
+extern const char* System_dirname(const char* str);
+extern const char* System_realpath(const char *path);
+extern const char* System_stringReplace(const char* str, const char* source, const char* target);
 char* _replace(const char* source_str,
                const char* search_str,
                const char* replace_str);
@@ -97,13 +101,14 @@ extern void SystemImpl__plotCallBack(threadData_t *threadData, int externalWindo
                                      const char* autoScale, const char* variables);
 extern double SystemImpl__time(void);
 extern int SystemImpl__directoryExists(const char* str);
+extern int SystemImpl__copyFile(const char* str_1, const char* str_2);
 extern int SystemImpl__createDirectory(const char *str);
 extern int SystemImpl__removeDirectory(const char *str);
-extern char* SystemImpl__readFileNoNumeric(const char* filename);
+extern const char* SystemImpl__readFileNoNumeric(const char* filename);
 extern double SystemImpl__getCurrentTime(void);
 extern int SystemImpl__unescapedStringLength(const char* str);
-extern char* SystemImpl__iconv(const char * str, const char *from, const char *to, int printError);
-extern char* SystemImpl__iconv__ascii(const char * str);
+extern const char* SystemImpl__iconv(const char * str, const char *from, const char *to, int printError);
+extern const char* SystemImpl__iconv__ascii(const char * str);
 extern void SystemImpl__initGarbageCollector(void);
 extern int SystemImpl__regularFileWritable(const char* str);
 
