@@ -30,6 +30,11 @@
 
 /* Stack overflow handling */
 
+#if defined(__linux__) && !defined(_GNU_SOURCE)
+#define _GNU_SOURCE 1
+/* for pthread_getattr_np */
+#endif
+
 #include "meta_modelica.h"
 
 void* mmc_getStacktraceMessages_threadData(threadData_t *threadData)
@@ -40,7 +45,7 @@ void* mmc_getStacktraceMessages_threadData(threadData_t *threadData)
   return threadData->localRoots[LOCAL_ROOT_STACK_OVERFLOW];
 }
 
-void* mmc_clearStacktraceMessages(threadData_t *threadData)
+void mmc_clearStacktraceMessages(threadData_t *threadData)
 {
   threadData->localRoots[LOCAL_ROOT_STACK_OVERFLOW] = 0;
 }
@@ -50,14 +55,9 @@ int mmc_hasStacktraceMessages(threadData_t *threadData)
   return threadData->localRoots[LOCAL_ROOT_STACK_OVERFLOW] != 0;
 }
 
-#if defined(linux) && !defined(_GNU_SOURCE)
-#define _GNU_SOURCE 1
-/* for pthread_getattr_np */
-#endif
-
 pthread_key_t mmc_stack_overflow_jumper;
 
-#if defined(linux) || defined(__APPLE_CC__)
+#if defined(__linux__) || defined(__APPLE_CC__)
 #include <stdlib.h>
 #include <stdio.h>
 #include <signal.h>
@@ -110,14 +110,35 @@ void mmc_setStacktraceMessages(int numSkip, int numFrames) {
   trace_size_skip = numSkip;
 }
 
+#define _OMC_LIT_OOM_data "Out of memory! Printed backtrace to stderr."
+static const MMC_DEFSTRINGLIT(_OMC_LIT_STRUCT_OOM,45,_OMC_LIT_OOM_data);
+static const MMC_DEFSTRUCTLIT(_OMC_LIT_STRUCT_OOM_LST,2,1) {MMC_REFSTRINGLIT(_OMC_LIT_STRUCT_OOM),MMC_REFSTRUCTLIT(mmc_nil)}};
+
 void mmc_setStacktraceMessages_threadData(threadData_t *threadData, int numSkip, int numFrames)
 {
   assert(numFrames > 0);
-  void **trace = (void**) GC_malloc_atomic(numFrames*sizeof(void*));
+  void **trace;
   char **messages;
   void *res;
   int i;
-  int trace_size = backtrace(trace, numFrames);
+  int trace_size;
+  size_t max_heap_size;
+
+  max_heap_size = omc_GC_get_max_heap_size();
+
+  if (0 != max_heap_size) {
+    /* Disable the memory limit so we can store the error-message */
+    GC_set_max_heap_size(0);
+  }
+
+  trace = (void**) GC_malloc_atomic(numFrames*sizeof(void*));
+  if (trace==0) {
+    mmc_setStacktraceMessages(numSkip, numFrames);
+    printStacktraceMessages();
+    threadData->localRoots[LOCAL_ROOT_STACK_OVERFLOW] = MMC_REFSTRINGLIT(_OMC_LIT_STRUCT_OOM_LST);
+    return;
+  }
+  trace_size = backtrace(trace, numFrames);
 
   res = mmc_mk_nil();
   messages = backtrace_symbols(trace, trace_size);
@@ -130,6 +151,9 @@ void mmc_setStacktraceMessages_threadData(threadData_t *threadData, int numSkip,
   }
   GC_free(trace);
   free(messages);
+  if (0 != max_heap_size) {
+    GC_set_max_heap_size(max_heap_size);
+  }
   threadData->localRoots[LOCAL_ROOT_STACK_OVERFLOW] = res;
 }
 
@@ -152,6 +176,10 @@ static void handler(int signo, siginfo_t *si, void *ptr)
   sigaction(SIGSEGV, &default_segv_action, 0);
 }
 
+#if defined(__APPLE_CC__)
+#include <sys/sysctl.h>
+#endif
+
 static void* getStackBase() {
   /* Warning: These functions are highly non-portable and are recommended to not be used.
    * We only tested them on Linux and OSX.
@@ -171,7 +199,16 @@ static void* getStackBase() {
 #else
   void* addr = pthread_get_stackaddr_np(self);
   size_t size = pthread_get_stacksize_np(self);
-  stackBottom = ((long)addr) - size;
+  stackBottom = (void*) (((long)addr) - size);
+  if (pthread_main_np()) {
+    char str[256];
+    size_t size = sizeof(str);
+    int ret = sysctlbyname("kern.osrelease", str, &size, NULL, 0);
+    if (0==ret && str[0]=='1' && str[1]=='3' && str[2]=='.') {
+      /* OSX Mavericks returns a wrong stack size... Assume default 8MB */
+      stackBottom = (void*) (((long)addr) - 8 * 1024 * 1024);
+    }
+  }
 #endif
   assert(size > 128*1024);
   return stackBottom + 64*1024;
@@ -219,3 +256,10 @@ void mmc_init_stackoverflow(threadData_t *threadData)
   threadData->stackBottom = 0x1; /* Dummy until Windows detects the stack bottom */
 }
 #endif
+
+void mmc_do_stackoverflow(threadData_t *threadData)
+{
+  /* Do the jump */
+  mmc_setStacktraceMessages_threadData(threadData, 1, MMC_SEGV_TRACE_NFRAMES);
+  longjmp(*threadData->mmc_stack_overflow_jumper, 1);
+}
