@@ -74,7 +74,7 @@
 #include <math.h>
 #include <string.h>
 
-int init_lambda_steps = 4;
+extern int init_lambda_steps = 3;
 
 /*! \fn void dumpInitializationStatus(DATA *data)
  *
@@ -89,8 +89,8 @@ void dumpInitialSolution(DATA *simData)
   const MODEL_DATA      *mData = simData->modelData;
   const SIMULATION_INFO *sInfo = simData->simulationInfo;
 
-  if (ACTIVE_STREAM(LOG_INIT))
-    printParameters(simData, LOG_INIT);
+  if (ACTIVE_STREAM(LOG_INIT_V))
+    printParameters(simData, LOG_INIT_V);
 
   if (!ACTIVE_STREAM(LOG_SOTI)) return;
   infoStreamPrint(LOG_SOTI, 1, "### SOLUTION OF THE INITIALIZATION ###");
@@ -203,9 +203,16 @@ static int symbolic_initialization(DATA *data, threadData_t *threadData)
     }
   }
 #endif
-  adaptiveGlobal = data->callback->useHomotopy == 2;
+  /* useHomotopy=1: global homotopy (equidistant lambda) */
+  if (data->callback->useHomotopy == 1 && omc_flag[FLAG_HOMOTOPY_ON_FIRST_TRY] != 1 && omc_flag[FLAG_NO_HOMOTOPY_ON_FIRST_TRY] != 1) {
+      omc_flag[FLAG_HOMOTOPY_ON_FIRST_TRY] = 1;
+      infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "Model contains homotopy operator: Use adaptive homotopy method to solve initialization problem. "
+                                            "To disable initialization with homotopy operator use \"-noHomotopyOnFirstTry\".");
+  }
+
+  adaptiveGlobal = data->callback->useHomotopy == 2;  /* new global homotopy approach (adaptive lambda) */
   solveWithGlobalHomotopy = homotopySupport
-                            && ((data->callback->useHomotopy == 1 && init_lambda_steps > 1) || adaptiveGlobal);
+                            && ((data->callback->useHomotopy == 1 && init_lambda_steps >= 1) || adaptiveGlobal);
 
 #if !defined(OMC_NDELAY_EXPRESSIONS) || OMC_NDELAY_EXPRESSIONS>0
   /* initial sample and delay before initial the system */
@@ -233,12 +240,12 @@ static int symbolic_initialization(DATA *data, threadData_t *threadData)
   MMC_TRY_INTERNAL(simulationJumpBuffer)
 #endif
     if (adaptiveGlobal && kinsol) {
-      infoStreamPrint(LOG_INIT, 0, "Automatically set -homotopyOnFirstTry, because trying without homotopy first is not supported for the adaptive global approach in combination with KINSOL.");
+      infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "Automatically set -homotopyOnFirstTry, because trying without homotopy first is not supported for the adaptive global approach in combination with KINSOL.");
     } else {
       if (adaptiveGlobal)
-        data->callback->useHomotopy = 1;
+        data->callback->useHomotopy = 1;  /* global homotopy (equidistant lambda) */
       data->simulationInfo->lambda = 1.0;
-      infoStreamPrint(LOG_INIT, 0, "Try to solve the initialization problem without homotopy first.");
+      infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "Try to solve the initialization problem without homotopy first.");
       data->callback->functionInitialEquations(data, threadData);
       solveWithGlobalHomotopy = 0;
     }
@@ -248,7 +255,7 @@ static int symbolic_initialization(DATA *data, threadData_t *threadData)
   MMC_CATCH_INTERNAL(simulationJumpBuffer)
 #endif
     if (adaptiveGlobal)
-      data->callback->useHomotopy = 2;
+      data->callback->useHomotopy = 2; /* new global homotopy approach (adaptive lambda) */
     if(solveWithGlobalHomotopy) {
       if (!kinsol)
         warningStreamPrint(LOG_ASSERT, 0, "Failed to solve the initialization problem without homotopy method. If homotopy is available the homotopy method is used now.");
@@ -268,16 +275,27 @@ static int symbolic_initialization(DATA *data, threadData_t *threadData)
     long step;
     char buffer[4096];
     double lambda;
+    int success = 0;
 
-    infoStreamPrint(LOG_INIT, 0, "Global homotopy with equidistant step size started.");
+    infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "Global homotopy with equidistant step size started.");
 
 #if !defined(OMC_NO_FILESYSTEM)
     const char sep[] = ",";
-    if(ACTIVE_STREAM(LOG_INIT))
+    if(ACTIVE_STREAM(LOG_INIT_HOMOTOPY))
     {
-      sprintf(buffer, "%s_equidistant_global_homotopy.csv", mData->modelFilePrefix);
-      infoStreamPrint(LOG_INIT, 0, "The homotopy path will be exported to %s.", buffer);
+      if (omc_flag[FLAG_OUTPUT_PATH]) { /* Add output path to file name */
+        sprintf(buffer, "%s/%s_equidistant_global_homotopy.csv", omc_flagValue[FLAG_OUTPUT_PATH], mData->modelFilePrefix);
+      }
+      else
+      {
+        sprintf(buffer, "%s_equidistant_global_homotopy.csv", mData->modelFilePrefix);
+      }
+      infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "The homotopy path will be exported to %s.", buffer);
       pFile = omc_fopen(buffer, "wt");
+      if (pFile == NULL)
+      {
+        throwStreamPrint(threadData, "Could not write to `%s`.", buffer);
+      }
       fprintf(pFile, "\"lambda\"");
       for(i=0; i<mData->nVariablesReal; ++i) {
         fprintf(pFile, "%s\"%s\"", sep, mData->realVarsData[i].info.name);
@@ -286,12 +304,16 @@ static int symbolic_initialization(DATA *data, threadData_t *threadData)
     }
 #endif
 
-    infoStreamPrint(LOG_INIT, 1, "homotopy process\n---------------------------");
-    for(step=0; step<init_lambda_steps; ++step)
+    infoStreamPrint(LOG_INIT_HOMOTOPY, 1, "homotopy process\n---------------------------");
+    /* try */
+#ifndef OMC_EMCC
+  MMC_TRY_INTERNAL(simulationJumpBuffer)
+#endif
+    for(step=0; step<=init_lambda_steps; ++step)
     {
-      data->simulationInfo->lambda = ((double)step)/(init_lambda_steps-1);
+      data->simulationInfo->lambda = ((double)step)/(init_lambda_steps);
       lambda = data->simulationInfo->lambda;
-      infoStreamPrint(LOG_INIT, 0, "homotopy parameter lambda = %g", lambda);
+      infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "homotopy parameter lambda = %g", lambda);
 
       if(data->simulationInfo->lambda > 1.0)
       {
@@ -300,27 +322,58 @@ static int symbolic_initialization(DATA *data, threadData_t *threadData)
       }
 
       if(0 == step)
-        data->callback->functionInitialEquations_lambda0(data, threadData);
+      {
+        if(data->callback->functionInitialEquations_lambda0 != NULL)
+        {
+          data->callback->functionInitialEquations_lambda0(data, threadData);
+        }
+        else
+        {
+          warningStreamPrint(LOG_INIT_HOMOTOPY, 0, "No initialEquation_lambda0 was generated. Using normal initial equation system with lambda=0 instead.");
+          data->callback->functionInitialEquations(data, threadData);
+        }
+      }
       else
+      {
         data->callback->functionInitialEquations(data, threadData);
+      }
 
-      infoStreamPrint(LOG_INIT, 0, "homotopy parameter lambda = %g done\n---------------------------", lambda);
+      infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "homotopy parameter lambda = %g done\n---------------------------", lambda);
 
 #if !defined(OMC_NO_FILESYSTEM)
-      if(ACTIVE_STREAM(LOG_INIT))
+      if(ACTIVE_STREAM(LOG_INIT_HOMOTOPY))
       {
         fprintf(pFile, "%.16g", lambda);
         for(i=0; i<mData->nVariablesReal; ++i)
+        {
           fprintf(pFile, "%s%.16g", sep, data->localData[0]->realVars[i]);
+        }
         fprintf(pFile, "\n");
       }
 #endif
     }
+    success = 1;
+    /* catch */
+#ifndef OMC_EMCC
+  MMC_CATCH_INTERNAL(simulationJumpBuffer)
+#endif
+    /* Error handling in case an assert was thrown */
+    if (!success)
+    {
+      messageClose(LOG_INIT_HOMOTOPY);
+#if !defined(OMC_NO_FILESYSTEM)
+      if(ACTIVE_STREAM(LOG_INIT_HOMOTOPY))
+        fclose(pFile);
+#endif
+      errorStreamPrint(LOG_ASSERT, 0, "Failed to solve the initialization problem with global homotopy with equidistant step size.");
+      throwStreamPrint(threadData, "Unable to solve initialization problem.");
+    }
+
     data->simulationInfo->homotopySteps += init_lambda_steps;
-    messageClose(LOG_INIT);
+    messageClose(LOG_INIT_HOMOTOPY);
 
 #if !defined(OMC_NO_FILESYSTEM)
-    if(ACTIVE_STREAM(LOG_INIT))
+    if(ACTIVE_STREAM(LOG_INIT_HOMOTOPY))
       fclose(pFile);
 #endif
   }
@@ -330,19 +383,27 @@ static int symbolic_initialization(DATA *data, threadData_t *threadData)
      use ADAPTIVE GLOBAL HOMOTOPY APPROACH. */
   if (adaptiveGlobal && solveWithGlobalHomotopy)
   {
-    infoStreamPrint(LOG_INIT, 0, "Global homotopy with adaptive step size started.");
-    infoStreamPrint(LOG_INIT, 1, "homotopy process\n---------------------------");
+    infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "Global homotopy with adaptive step size started.");
+    infoStreamPrint(LOG_INIT_HOMOTOPY, 1, "homotopy process\n---------------------------");
 
     // Solve lambda0-DAE
     data->simulationInfo->lambda = 0;
-    infoStreamPrint(LOG_INIT, 0, "solve simplified lambda0-DAE");
-    data->callback->functionInitialEquations_lambda0(data, threadData);
-    infoStreamPrint(LOG_INIT, 0, "solving simplified lambda0-DAE done\n---------------------------");
+    infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "solve simplified lambda0-DAE");
+    if(data->callback->functionInitialEquations_lambda0 != NULL)
+    {
+      data->callback->functionInitialEquations_lambda0(data, threadData);
+    }
+    else
+    {
+      warningStreamPrint(LOG_INIT_HOMOTOPY, 0, "No initialEquation_lambda0 was generated. Using normal initial equation system with lambda=0 instead.");
+      data->callback->functionInitialEquations(data, threadData);
+    }
+    infoStreamPrint(LOG_INIT_HOMOTOPY, 0, "solving simplified lambda0-DAE done\n---------------------------");
 
     // Run along the homotopy path and solve the actual system
     data->callback->functionInitialEquations(data, threadData);
 
-    messageClose(LOG_INIT);
+    messageClose(LOG_INIT_HOMOTOPY);
   }
 
   storeRelations(data);
@@ -461,7 +522,7 @@ int importStartValues(DATA *data, threadData_t *threadData, const char *pInitFil
       }
       if(pVar) {
         omc_matlab4_val(&(mData->realVarsData[i].attribute.start), &reader, pVar, initTime);
-        infoStreamPrint(LOG_INIT, 0, "| %s(start=%g)", mData->realVarsData[i].info.name, mData->realVarsData[i].attribute.start);
+        infoStreamPrint(LOG_INIT_V, 0, "| %s(start=%g)", mData->realVarsData[i].info.name, mData->realVarsData[i].attribute.start);
       } else if((strlen(mData->realVarsData[i].info.name) > 0) &&
               (mData->realVarsData[i].info.name[0] != '$') &&
               (strncmp(mData->realVarsData[i].info.name, "der($", 5) != 0)) {
@@ -483,7 +544,7 @@ int importStartValues(DATA *data, threadData_t *threadData, const char *pInitFil
       if(pVar) {
         omc_matlab4_val(&(mData->realParameterData[i].attribute.start), &reader, pVar, initTime);
         data->simulationInfo->realParameter[i] = mData->realParameterData[i].attribute.start;
-        infoStreamPrint(LOG_INIT, 0, "| %s(start=%g)", mData->realParameterData[i].info.name, mData->realParameterData[i].attribute.start);
+        infoStreamPrint(LOG_INIT_V, 0, "| %s(start=%g)", mData->realParameterData[i].info.name, mData->realParameterData[i].attribute.start);
       } else {
         warningStreamPrint(LOG_INIT, 0, "unable to import real parameter %s from given file", mData->realParameterData[i].info.name);
       }
@@ -501,7 +562,7 @@ int importStartValues(DATA *data, threadData_t *threadData, const char *pInitFil
 
       if(pVar) {
         omc_matlab4_val(&(mData->realParameterData[i].attribute.start), &reader, pVar, initTime);
-        infoStreamPrint(LOG_INIT, 0, "| %s(start=%g)", mData->realParameterData[i].info.name, mData->realParameterData[i].attribute.start);
+        infoStreamPrint(LOG_INIT_V, 0, "| %s(start=%g)", mData->realParameterData[i].info.name, mData->realParameterData[i].attribute.start);
       } else {
         warningStreamPrint(LOG_INIT, 0, "unable to import real parameter %s from given file", mData->realParameterData[i].info.name);
       }
@@ -522,7 +583,7 @@ int importStartValues(DATA *data, threadData_t *threadData, const char *pInitFil
         omc_matlab4_val(&value, &reader, pVar, initTime);
         mData->integerParameterData[i].attribute.start = (modelica_integer)value;
         data->simulationInfo->integerParameter[i] = (modelica_integer)value;
-        infoStreamPrint(LOG_INIT, 0, "| %s(start=%ld)", mData->integerParameterData[i].info.name, mData->integerParameterData[i].attribute.start);
+        infoStreamPrint(LOG_INIT_V, 0, "| %s(start=%ld)", mData->integerParameterData[i].info.name, mData->integerParameterData[i].attribute.start);
       } else {
         warningStreamPrint(LOG_INIT, 0, "unable to import integer parameter %s from given file", mData->integerParameterData[i].info.name);
       }
@@ -542,7 +603,7 @@ int importStartValues(DATA *data, threadData_t *threadData, const char *pInitFil
         omc_matlab4_val(&value, &reader, pVar, initTime);
         mData->booleanParameterData[i].attribute.start = (modelica_boolean)value;
         data->simulationInfo->booleanParameter[i] = (modelica_boolean)value;
-        infoStreamPrint(LOG_INIT, 0, "| %s(start=%s)", mData->booleanParameterData[i].info.name, mData->booleanParameterData[i].attribute.start ? "true" : "false");
+        infoStreamPrint(LOG_INIT_V, 0, "| %s(start=%s)", mData->booleanParameterData[i].info.name, mData->booleanParameterData[i].attribute.start ? "true" : "false");
       } else {
         warningStreamPrint(LOG_INIT, 0, "unable to import boolean parameter %s from given file", mData->booleanParameterData[i].info.name);
       }
@@ -635,6 +696,8 @@ int initialization(DATA *data, threadData_t *threadData, const char* pInitMethod
     data->callback->updateBoundVariableAttributes(data, threadData);
   }
 
+  data->callback->function_initSpatialDistribution(data, threadData);
+
   /* update static data of linear/non-linear system solvers */
   updateStaticDataOfLinearSystems(data, threadData);
   updateStaticDataOfNonlinearSystems(data, threadData);
@@ -730,6 +793,7 @@ int initialization(DATA *data, threadData_t *threadData, const char* pInitMethod
 
   initSample(data, threadData, data->simulationInfo->startTime, data->simulationInfo->stopTime);
   data->callback->function_storeDelayed(data, threadData);
+  data->callback->function_storeSpatialDistribution(data, threadData);
   data->callback->function_updateRelations(data, threadData, 1);
   initSynchronous(data, threadData, data->simulationInfo->startTime);
 
