@@ -33,7 +33,7 @@
 
 #include "omc_config.h"
 
-#ifdef WITH_UMFPACK
+#ifdef WITH_SUITESPARSE
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
@@ -41,16 +41,17 @@
 #include "simulation_data.h"
 #include "simulation/simulation_info_json.h"
 #include "util/omc_error.h"
+#include "util/parallel_helper.h"
+#include "omc_math.h"
 #include "util/varinfo.h"
 #include "model_help.h"
 
 #include "linearSystem.h"
 #include "linearSolverUmfpack.h"
 
-
 void printMatrixCSC(int* Ap, int* Ai, double* Ax, int n);
 void printMatrixCSR(int* Ap, int* Ai, double* Ax, int n);
-int solveSingularSystem(LINEAR_SYSTEM_DATA* systemData);
+int solveSingularSystem(LINEAR_SYSTEM_DATA* systemData, double* aux_x);
 
 /*! \fn allocate memory for linear system solver UmfPack
  *
@@ -137,25 +138,28 @@ int getAnalyticalJacobianUmfPack(DATA* data, threadData_t *threadData, int sysNu
   LINEAR_SYSTEM_DATA* systemData = &(((DATA*)data)->simulationInfo->linearSystemData[sysNumber]);
 
   const int index = systemData->jacobianIndex;
+  ANALYTIC_JACOBIAN* jacobian = systemData->parDynamicData[omc_get_thread_num()].jacobian;
+  ANALYTIC_JACOBIAN* parentJacobian = systemData->parDynamicData[omc_get_thread_num()].parentJacobian;
+
   int nth = 0;
-  int nnz = data->simulationInfo->analyticJacobians[index].sparsePattern.numberOfNoneZeros;
+  int nnz = jacobian->sparsePattern->numberOfNoneZeros;
 
-  for(i=0; i < data->simulationInfo->analyticJacobians[index].sizeRows; i++)
+  for(i=0; i < jacobian->sizeRows; i++)
   {
-    data->simulationInfo->analyticJacobians[index].seedVars[i] = 1;
+    jacobian->seedVars[i] = 1;
 
-    ((systemData->analyticalJacobianColumn))(data, threadData);
+    ((systemData->analyticalJacobianColumn))(data, threadData, jacobian, parentJacobian);
 
-    for(j = 0; j < data->simulationInfo->analyticJacobians[index].sizeCols; j++)
+    for(j = 0; j < jacobian->sizeCols; j++)
     {
-      if(data->simulationInfo->analyticJacobians[index].seedVars[j] == 1)
+      if(jacobian->seedVars[j] == 1)
       {
-        ii = data->simulationInfo->analyticJacobians[index].sparsePattern.leadindex[j];
-        while(ii < data->simulationInfo->analyticJacobians[index].sparsePattern.leadindex[j+1])
+        ii = jacobian->sparsePattern->leadindex[j];
+        while(ii < jacobian->sparsePattern->leadindex[j+1])
         {
-          l  = data->simulationInfo->analyticJacobians[index].sparsePattern.index[ii];
-          /* infoStreamPrint(LOG_LS_V, 0, "set on Matrix A (%d, %d)(%d) = %f", i, l, nth, -data->simulationInfo->analyticJacobians[index].resultVars[l]); */
-          systemData->setAElement(i, l, -data->simulationInfo->analyticJacobians[index].resultVars[l], nth, (void*) systemData, threadData);
+          l  = jacobian->sparsePattern->index[ii];
+          /* infoStreamPrint(LOG_LS_V, 0, "set on Matrix A (%d, %d)(%d) = %f", i, l, nth, -jacobian->resultVars[l]); */
+          systemData->setAElement(i, l, -jacobian->resultVars[l], nth, (void*) systemData, threadData);
           nth++;
           ii++;
         };
@@ -163,7 +167,7 @@ int getAnalyticalJacobianUmfPack(DATA* data, threadData_t *threadData, int sysNu
     };
 
     /* de-activate seed variable for the corresponding color */
-    data->simulationInfo->analyticJacobians[index].seedVars[i] = 0;
+    jacobian->seedVars[i] = 0;
   }
 
   return 0;
@@ -189,52 +193,50 @@ static int wrapper_fvec_umfpack(double* x, double* f, void** data, int sysNumber
  * author: kbalzereit, wbraun
  */
 int
-solveUmfPack(DATA *data, threadData_t *threadData, int sysNumber)
+solveUmfPack(DATA *data, threadData_t *threadData, int sysNumber, double* aux_x)
 {
   void *dataAndThreadData[2] = {data, threadData};
   LINEAR_SYSTEM_DATA* systemData = &(data->simulationInfo->linearSystemData[sysNumber]);
-  DATA_UMFPACK* solverData = (DATA_UMFPACK*)systemData->solverData;
+  DATA_UMFPACK* solverData = (DATA_UMFPACK*)systemData->parDynamicData[omc_get_thread_num()].solverData[0];
+  _omc_scalar residualNorm = 0;
 
   int i, j, status = UMFPACK_OK, success = 0, ni=0, n = systemData->size, eqSystemNumber = systemData->equationIndex, indexes[2] = {1,eqSystemNumber};
   int casualTearingSet = systemData->strictTearingFunctionCall != NULL;
   double tmpJacEvalTime;
+  int reuseMatrixJac = (data->simulationInfo->currentContext == CONTEXT_SYM_JACOBIAN && data->simulationInfo->currentJacobianEval > 0);
 
   infoStreamPrintWithEquationIndexes(LOG_LS, 0, indexes, "Start solving Linear System %d (size %d) at time %g with UMFPACK Solver",
    eqSystemNumber, (int) systemData->size,
    data->localData[0]->timeValue);
 
-
   rt_ext_tp_tick(&(solverData->timeClock));
   if (0 == systemData->method)
   {
-    /* set A matrix */
-    solverData->Ap[0] = 0;
-    systemData->setA(data, threadData, systemData);
-    solverData->Ap[solverData->n_row] = solverData->nnz;
-
-    if (ACTIVE_STREAM(LOG_LS_V))
-    {
-      infoStreamPrint(LOG_LS_V, 1, "Matrix A");
-      printMatrixCSR(solverData->Ap, solverData->Ai, solverData->Ax, n);
-      messageClose(LOG_LS_V);
+    if (!reuseMatrixJac){
+      /* set A matrix */
+      solverData->Ap[0] = 0;
+      systemData->setA(data, threadData, systemData);
+      solverData->Ap[solverData->n_row] = solverData->nnz;
     }
 
     /* set b vector */
     systemData->setb(data, threadData, systemData);
   } else {
 
-    solverData->Ap[0] = 0;
-    /* calculate jacobian -> matrix A*/
-    if(systemData->jacobianIndex != -1){
-      getAnalyticalJacobianUmfPack(data, threadData, sysNumber);
-    } else {
-      assertStreamPrint(threadData, 1, "jacobian function pointer is invalid" );
+    if (!reuseMatrixJac){
+      solverData->Ap[0] = 0;
+      /* calculate jacobian -> matrix A*/
+      if(systemData->jacobianIndex != -1){
+        getAnalyticalJacobianUmfPack(data, threadData, sysNumber);
+      } else {
+        assertStreamPrint(threadData, 1, "jacobian function pointer is invalid" );
+      }
+      solverData->Ap[solverData->n_row] = solverData->nnz;
     }
-    solverData->Ap[solverData->n_row] = solverData->nnz;
 
     /* calculate vector b (rhs) */
-    memcpy(solverData->work, systemData->x, sizeof(double)*solverData->n_row);
-    wrapper_fvec_umfpack(solverData->work, systemData->b, dataAndThreadData, sysNumber);
+    memcpy(solverData->work, aux_x, sizeof(double)*solverData->n_row);
+    wrapper_fvec_umfpack(solverData->work, systemData->parDynamicData[omc_get_thread_num()].b, dataAndThreadData, sysNumber);
   }
   tmpJacEvalTime = rt_ext_tp_tock(&(solverData->timeClock));
   systemData->jacobianTime += tmpJacEvalTime;
@@ -244,7 +246,7 @@ solveUmfPack(DATA *data, threadData_t *threadData, int sysNumber)
   {
     infoStreamPrint(LOG_LS_V, 1, "Old solution x:");
     for(i = 0; i < solverData->n_row; ++i)
-      infoStreamPrint(LOG_LS_V, 0, "[%d] %s = %g", i+1, modelInfoGetEquation(&data->modelData->modelDataXml,eqSystemNumber).vars[i], systemData->x[i]);
+      infoStreamPrint(LOG_LS_V, 0, "[%d] %s = %g", i+1, modelInfoGetEquation(&data->modelData->modelDataXml,eqSystemNumber).vars[i], aux_x[i]);
     messageClose(LOG_LS_V);
 
     infoStreamPrint(LOG_LS_V, 1, "Matrix A n_rows = %d", solverData->n_row);
@@ -256,28 +258,33 @@ solveUmfPack(DATA *data, threadData_t *threadData, int sysNumber)
     }
     messageClose(LOG_LS_V);
 
-    for (i=0; i<solverData->n_row; i++)
-      infoStreamPrint(LOG_LS_V, 0, "b[%d] = %e", i, systemData->b[i]);
+    for (i=0; i<solverData->n_row; i++) {
+      // ToDo Rework stream prints like this one to work in parallel regions
+      infoStreamPrint(LOG_LS_V, 0, "b[%d] = %e", i, systemData->parDynamicData[omc_get_thread_num()].b[i]);
+    }
   }
   rt_ext_tp_tick(&(solverData->timeClock));
 
   /* symbolic pre-ordering of A to reduce fill-in of L and U */
-  if (0 == solverData->numberSolving)
-  {
+  if (0 == solverData->numberSolving) {
     status = umfpack_di_symbolic(solverData->n_col, solverData->n_row, solverData->Ap, solverData->Ai, solverData->Ax, &(solverData->symbolic), solverData->control, solverData->info);
   }
 
   /* compute the LU factorization of A */
-  if (0 == status){
-    umfpack_di_free_numeric(&(solverData->numeric));
-    status = umfpack_di_numeric(solverData->Ap, solverData->Ai, solverData->Ax, solverData->symbolic, &(solverData->numeric), solverData->control, solverData->info);
+  /* if reuseMatrixJac use also previous factorization */
+  if (!reuseMatrixJac)
+  {
+    if (0 == status){
+      umfpack_di_free_numeric(&(solverData->numeric));
+      status = umfpack_di_numeric(solverData->Ap, solverData->Ai, solverData->Ax, solverData->symbolic, &(solverData->numeric), solverData->control, solverData->info);
+    }
   }
 
   if (0 == status){
     if (1 == systemData->method){
-      status = umfpack_di_wsolve(UMFPACK_A, solverData->Ap, solverData->Ai, solverData->Ax, systemData->x, systemData->b, solverData->numeric, solverData->control, solverData->info, solverData->Wi, solverData->W);
+      status = umfpack_di_wsolve(UMFPACK_A, solverData->Ap, solverData->Ai, solverData->Ax, aux_x, systemData->parDynamicData[omc_get_thread_num()].b, solverData->numeric, solverData->control, solverData->info, solverData->Wi, solverData->W);
     } else {
-      status = umfpack_di_wsolve(UMFPACK_Aat, solverData->Ap, solverData->Ai, solverData->Ax, systemData->x, systemData->b, solverData->numeric, solverData->control, solverData->info, solverData->Wi, solverData->W);
+      status = umfpack_di_wsolve(UMFPACK_Aat, solverData->Ap, solverData->Ai, solverData->Ax, aux_x, systemData->parDynamicData[omc_get_thread_num()].b, solverData->numeric, solverData->control, solverData->info, solverData->Wi, solverData->W);
     }
   }
 
@@ -286,7 +293,7 @@ solveUmfPack(DATA *data, threadData_t *threadData, int sysNumber)
   }
   else if ((status == UMFPACK_WARNING_singular_matrix) && (casualTearingSet==0))
   {
-    if (!solveSingularSystem(systemData))
+    if (!solveSingularSystem(systemData, aux_x))
     {
       success = 1;
     }
@@ -298,21 +305,33 @@ solveUmfPack(DATA *data, threadData_t *threadData, int sysNumber)
     if (1 == systemData->method){
       /* take the solution */
       for(i = 0; i < solverData->n_row; ++i)
-        systemData->x[i] += solverData->work[i];
+        aux_x[i] += solverData->work[i];
 
       /* update inner equations */
-      wrapper_fvec_umfpack(systemData->x, solverData->work, dataAndThreadData, sysNumber);
+      wrapper_fvec_umfpack(aux_x, solverData->work, dataAndThreadData, sysNumber);
+      residualNorm = _omc_gen_euclideanVectorNorm(solverData->work, solverData->n_row);
+
+      if ((isnan(residualNorm)) || (residualNorm>1e-4)){
+        warningStreamPrint(LOG_LS, 0,
+            "Failed to solve linear system of equations (no. %d) at time %f. Residual norm is %.15g.",
+            (int)systemData->equationIndex, data->localData[0]->timeValue, residualNorm);
+        success = 0;
+      }
     } else {
       /* the solution is automatically in x */
     }
 
     if (ACTIVE_STREAM(LOG_LS_V))
     {
-      infoStreamPrint(LOG_LS_V, 1, "Solution x:");
+      if (1 == systemData->method) {
+        infoStreamPrint(LOG_LS_V, 1, "Residual Norm %.15g of solution x:", residualNorm);
+      } else {
+        infoStreamPrint(LOG_LS_V, 1, "Solution x:");
+      }
       infoStreamPrint(LOG_LS_V, 0, "System %d numVars %d.", eqSystemNumber, modelInfoGetEquation(&data->modelData->modelDataXml,eqSystemNumber).numVar);
 
       for(i = 0; i < systemData->size; ++i)
-        infoStreamPrint(LOG_LS_V, 0, "[%d] %s = %g", i+1, modelInfoGetEquation(&data->modelData->modelDataXml,eqSystemNumber).vars[i], systemData->x[i]);
+        infoStreamPrint(LOG_LS_V, 0, "[%d] %s = %g", i+1, modelInfoGetEquation(&data->modelData->modelDataXml,eqSystemNumber).vars[i], aux_x[i]);
 
       messageClose(LOG_LS_V);
     }
@@ -351,10 +370,9 @@ solveUmfPack(DATA *data, threadData_t *threadData, int sysNumber)
  *
  * author: kbalzereit, wbraun
  */
-int solveSingularSystem(LINEAR_SYSTEM_DATA* systemData)
+int solveSingularSystem(LINEAR_SYSTEM_DATA* systemData, double* aux_x)
 {
-
-  DATA_UMFPACK* solverData = (DATA_UMFPACK*) systemData->solverData;
+  DATA_UMFPACK* solverData = (DATA_UMFPACK*) systemData->parDynamicData[omc_get_thread_num()].solverData[0];
   double *Ux, *Rs, r_ii, *b, sum, *y, *z;
   int *Up, *Ui, *Q, do_recip, rank = 0, current_rank, current_unz, i, j, k, l,
       success = 0, status, stop = 0;
@@ -393,14 +411,13 @@ int solveSingularSystem(LINEAR_SYSTEM_DATA* systemData)
   {
     for (i = 0; i < solverData->n_row; i++)
     {
-      b[i] = systemData->b[i] / Rs[i];
+      b[i] = systemData->parDynamicData[omc_get_thread_num()].b[i] / Rs[i];
     }
   }
   else
   {
-    for (i = 0; i < solverData->n_row; i++)
-    {
-      b[i] = systemData->b[i] * Rs[i];
+    for (i = 0; i < solverData->n_row; i++) {
+      b[i] = systemData->parDynamicData[omc_get_thread_num()].b[i] * Rs[i];
     }
   }
 
@@ -540,7 +557,7 @@ int solveSingularSystem(LINEAR_SYSTEM_DATA* systemData)
   /* x = Q^T * z */
   for (i = 0; i < solverData->n_col; i++)
   {
-    systemData->x[Q[i]] = z[i];
+    aux_x[Q[i]] = z[i];
   }
 
   /* free all used memory */
