@@ -75,8 +75,8 @@ typedef struct DATA_HOMOTOPY
 {
   int initialized; /* 1 = initialized, else = 0*/
 
-  int n; /* dimension; n == size */
-  int m; /* dimension: m == size+1 */
+  size_t n; /* dimension; n == size */
+  size_t m; /* dimension: m == size+1 */
 
   double xtol_sqrd; /* tolerance for updating solution vector */
   double ftol_sqrd; /* tolerance for accepting accuracy */
@@ -163,7 +163,7 @@ typedef struct DATA_HOMOTOPY
  *  allocate memory for nonlinear system solver
  *  \author bbachmann
  */
-int allocateHomotopyData(int size, void** voiddata)
+int allocateHomotopyData(size_t size, void** voiddata)
 {
   DATA_HOMOTOPY* data = (DATA_HOMOTOPY*) malloc(sizeof(DATA_HOMOTOPY));
 
@@ -681,9 +681,10 @@ double vecScalarProd(int n, double *a, double *b)
 void matVecMult(int n, int m, double *A, double *b, double *c)
 {
   int i, j;
-  for (i=0;i<n;i++) {
+  for (i=0;i<n;i++)
     c[i] = 0.0;
-    for (j=0;j<m;j++)
+  for (j=0;j<m;j++) {
+    for (i=0;i<n;i++)
       c[i] += A[i+j*(m-1)]*b[j];
   }
 }
@@ -692,9 +693,10 @@ void matVecMult(int n, int m, double *A, double *b, double *c)
 void matVecMultAbs(int n, int m, double *A, double *b, double *c)
 {
   int i, j;
-  for (i=0;i<n;i++) {
+  for (i=0;i<n;i++)
     c[i] = 0.0;
-    for (j=0;j<m;j++)
+  for (j=0;j<m;j++) {
+    for (i=0;i<n;i++)
       c[i] += fabs(A[i+j*(m-1)]*b[j]);
   }
 }
@@ -703,9 +705,10 @@ void matVecMultAbs(int n, int m, double *A, double *b, double *c)
 void matVecMultBB(int n, double *A, double *b, double *c)
 {
   int i, j;
-  for (i=0;i<n;i++) {
+  for (i=0;i<n;i++)
     c[i] = 0.0;
-    for (j=0;j<n;j++)
+  for (j=0;j<n;j++) {
+    for (i=0;i<n;i++)
       c[i] += A[i+j*n]*b[j];
   }
 }
@@ -714,10 +717,11 @@ void matVecMultBB(int n, double *A, double *b, double *c)
 void matVecMultAbsBB(int n, double *A, double *b, double *c)
 {
   int i, j;
-  for (i=0;i<n;i++) {
+  for (i=0;i<n;i++)
     c[i] = 0.0;
-    for (j=0;j<n;j++)
-       c[i] += fabs(A[i+j*n]*b[j]);
+  for (j=0;j<n;j++) {
+    for (i=0;i<n;i++)
+      c[i] += fabs(A[i+j*n]*b[j]);
   }
 }
 
@@ -725,9 +729,8 @@ void matVecMultAbsBB(int n, double *A, double *b, double *c)
 void matAddBB(int n, double* A, double* B, double* C)
 {
   int i, j;
-
-  for (i=0;i<n;i++) {
-    for (j=0;j<n+1;j++)
+  for (j=0;j<n+1;j++) {
+    for (i=0;i<n;i++)
       C[i + j*n] = A[i + j*n] + B[i + j*n];
   }
 }
@@ -736,9 +739,8 @@ void matAddBB(int n, double* A, double* B, double* C)
 void matDiffBB(int n, double* A, double* B, double* C)
 {
   int i, j;
-
-  for (i=0;i<n;i++) {
-    for (j=0;j<n;j++)
+  for (j=0;j<n;j++) {
+    for (i=0;i<n;i++)
       C[i + j*n] = A[i + j*n] - B[i + j*n];
   }
 }
@@ -746,39 +748,50 @@ void matDiffBB(int n, double* A, double* B, double* C)
 /* Matrix has dimension [n x m] */
 void scaleMatrixRows(int n, int m, double *A)
 {
-  const double delta = sqrt(DBL_EPSILON);
+  const double delta = 0; /* This might be changed to sqrt(DBL_EPSILON) */
   int i, j;
-  double rowMax;
-  for (i=0;i<n;i++) {
-    rowMax = 0; /* This might be changed to delta */
-    for (j=0;j<n;j++) {
-      if (fabs(A[i+j*(m-1)]) > rowMax) {
-         rowMax = fabs(A[i+j*(m-1)]);
+  double* rowsMax = (double*) calloc(n,sizeof(double));
+
+  for (i=0;i<n;i++)
+    rowsMax[i] = 0;
+
+  /* find maximum of each row */
+  for (j=0;j<n;j++) {
+    for (i=0;i<n;i++) {
+      if (fabs(A[i+j*(m-1)]) > rowsMax[i]) {
+         rowsMax[i] = fabs(A[i+j*(m-1)]);
       }
     }
-    if (rowMax>0) {
-      for (j=0;j<m;j++)
-        A[i+j*(m-1)] /= rowMax;
-    }
   }
+
+  /* remove zero normailzation */
+  for (i=0;i<n;i++) {
+    if (rowsMax[i] <= delta)
+      rowsMax[i] = 1.0;
+  }
+
+  /* scale matrix */
+  for (j=0;j<m;j++) {
+    for (i=0;i<n;i++)
+      A[i+j*(m-1)] /= rowsMax[i];
+  }
+
+  free(rowsMax);
 }
 
 /* Build the newton matrix for the corrector step with orthogonal backtrace strategy */
 void orthogonalBacktraceMatrix(DATA_HOMOTOPY* solverData, double* hJac, double* hvec, double* v, double* hJac2, int n, int m)
 {
   int i, j;
-  for (i=0; i<n; i++)
-  {
-    for (j=0; j<m; j++)
-    {
+  for (j=0; j<m; j++) {
+    for (i=0; i<n; i++) {
       hJac2[i + j*m] = hJac[i + j*(m-1)];
     }
-    hJac2[i + m*m] = hvec[i];
-  }
-  for (j=0; j<m; j++)
-  {
     hJac2[n + j*m] = v[j];
   }
+  for (i=0; i<n; i++) {
+    hJac2[i + m*m] = hvec[i];
+  }
   hJac2[n + m*m] = 0;
 }
 
@@ -1315,6 +1328,7 @@ static int newtonAlgorithm(DATA_HOMOTOPY* solverData, double* x)
   int firstrun;
   int constraintViolated;
   int solverinfo = 0;
+  int lastWasGood = 0; /* boolean, keeps track of previous x */
 
   int assert = 1;
   threadData_t *threadData = solverData->threadData;
@@ -1502,21 +1516,24 @@ static int newtonAlgorithm(DATA_HOMOTOPY* solverData, double* x)
         lambda = lambda1;
       }
     }
-    /* updating x, fvec, error_f_sqrd */
-    /* event. swapPointer(&x, &(solverData->x1)); */
-    vecCopy(solverData->n, solverData->x1, x);
 
     /* Calculate different error measurements */
     vecDivScaling(solverData->n, solverData->f1, solverData->resScaling, solverData->fvecScaled);
     debugVectorDouble(LOG_NLS_V,"function values:",solverData->f1, n);
     debugVectorDouble(LOG_NLS_V,"scaled function values:",solverData->fvecScaled, n);
 
+    /* update delta_x_sqrd, error_f_sqrd */
     vecDivScaling(solverData->n, solverData->dy0, solverData->xScaling, solverData->dxScaled);
     delta_x_sqrd        = vec2NormSqrd(solverData->n, solverData->dy0);
     delta_x_sqrd_scaled = vec2NormSqrd(solverData->n, solverData->dxScaled);
+
+    error_f_old = error_f_sqrd;
     error_f_sqrd        = vec2NormSqrd(solverData->n, solverData->f1);
     error_f_sqrd_scaled = vec2NormSqrd(solverData->n, solverData->fvecScaled);
 
+    countNegativeSteps += (error_f_sqrd > 10*error_f_old);
+    lastWasGood = error_f_sqrd >= error_f_old;
+
 
     /* debug information */
     debugString(LOG_NLS_V, "error measurements:");
@@ -1527,9 +1544,6 @@ static int newtonAlgorithm(DATA_HOMOTOPY* solverData, double* x)
     debugDouble(LOG_NLS_V, "error_f_scaled =", sqrt(error_f_sqrd_scaled));
     debugDouble(LOG_NLS_V, "newtonFTol          =", sqrt(solverData->ftol_sqrd));
 
-    countNegativeSteps += (error_f_sqrd > 10*error_f_old);
-    error_f_old = error_f_sqrd;
-
 #if !defined(OMC_MINIMAL_RUNTIME)
     if (solverData->data->simulationInfo->nlsCsvInfomation){
       print_csvLineIterStats(((struct csvStats*) nonlinsys->csvData)->iterStats,
@@ -1558,6 +1572,16 @@ static int newtonAlgorithm(DATA_HOMOTOPY* solverData, double* x)
     {
       solverData->info = 1;
 
+      /* reject new x if old x is as good, for stability (see issue #6419) */
+      if (lastWasGood)
+      {
+        debugString(LOG_NLS_V, "Note: newton solver rejected last x because previous was as good");
+      }
+      else
+      {
+        vecCopy(solverData->n, solverData->x1, x);
+      }
+
       /* debug information */
       debugString(LOG_NLS_V, "NEWTON SOLVER DID CONVERGE TO A SOLUTION!!!");
       printUnknowns(LOG_NLS_V, solverData);
@@ -1619,6 +1643,9 @@ static int newtonAlgorithm(DATA_HOMOTOPY* solverData, double* x)
 #ifndef OMC_EMCC
     MMC_TRY_INTERNAL(simulationJumpBuffer)
 #endif
+    /* updating x */
+    vecCopy(solverData->n, solverData->x1, x);
+
     /* calculate jacobian and function values (both stored in fJac, last column is fvec) */
     solverData->fJac_f(solverData, x, solverData->fJac);
     assert = 0;
@@ -2070,8 +2097,8 @@ static int homotopyAlgorithm(DATA_HOMOTOPY* solverData, double *x)
 
   debugString(LOG_NLS_HOMOTOPY, "HOMOTOPY ALGORITHM SUCCEEDED");
   if (solverData->initHomotopy) {
-    solverData->data->simulationInfo->homotopySteps += numSteps+1;
-    debugInt(LOG_INIT_HOMOTOPY, "Total number of lambda steps for this homotopy loop:", numSteps+1);
+    solverData->data->simulationInfo->homotopySteps += numSteps;
+    debugInt(LOG_INIT_HOMOTOPY, "Total number of lambda steps for this homotopy loop:", numSteps);
   }
   debugString(LOG_NLS_HOMOTOPY, "======================================================");
   solverData->info = 1;
