@@ -65,6 +65,8 @@ DLLExport extern void (*omc_assert_withEquationIndexes)(threadData_t*,FILE_INFO,
 DLLExport extern void (*omc_assert_warning_withEquationIndexes)(FILE_INFO, const int*, const char*, ...);
 
 void initDumpSystem();
+void deactivateLogging();
+void reactivateLogging();
 void omc_assert_function(threadData_t*,FILE_INFO info, const char *msg, ...) __attribute__ ((noreturn));
 void omc_assert_warning_function(FILE_INFO info,  const char *msg, ...);
 void omc_terminate_function(FILE_INFO info, const char *msg, ...);
@@ -89,6 +91,8 @@ enum LOG_STREAM
   LOG_EVENTS,
   LOG_EVENTS_V,
   LOG_INIT,
+  LOG_INIT_HOMOTOPY,
+  LOG_INIT_V,
   LOG_IPOPT,
   LOG_IPOPT_FULL,
   LOG_IPOPT_JAC,
@@ -111,9 +115,11 @@ enum LOG_STREAM
   LOG_SOLVER_V,
   LOG_SOLVER_CONTEXT,
   LOG_SOTI,
+  LOG_SPATIALDISTR,
   LOG_STATS,
   LOG_STATS_V,
   LOG_SUCCESS,
+  LOG_SYNCHRONOUS,
 #ifdef USE_DEBUG_TRACE
   LOG_TRACE,
 #endif
