@@ -38,6 +38,9 @@ const char *FLAG_NAME[FLAG_MAX+1] = {
   /* FLAG_CLOCK */                        "clock",
   /* FLAG_CPU */                          "cpu",
   /* FLAG_CSV_OSTEP */                    "csvOstep",
+  /* FLAG_CVODE_ITER */                   "cvodeNonlinearSolverIteration",
+  /* FLAG_CVODE_LMM */                    "cvodeLinearMultistepMethod",
+  /* FLAG_DATA_RECONCILE_Cx */            "cx",
   /* FLAG_DAE_MODE */                     "daeMode",
   /* FLAG_DELTA_X_LINEARIZE */            "deltaXLinearize",
   /* FLAG_DELTA_X_SOLVER */               "deltaXSolver",
@@ -56,6 +59,7 @@ const char *FLAG_NAME[FLAG_MAX+1] = {
   /* FLAG_HOMOTOPY_MAX_TRIES */           "homMaxTries",
   /* FLAG_HOMOTOPY_NEG_START_DIR */       "homNegStartDir",
   /* FLAG_HOMOTOPY_ON_FIRST_TRY */        "homotopyOnFirstTry",
+  /* FLAG_NO_HOMOTOPY_ON_FIRST_TRY */     "noHomotopyOnFirstTry",
   /* FLAG_HOMOTOPY_TAU_DEC_FACTOR */      "homTauDecFac",
   /* FLAG_HOMOTOPY_TAU_DEC_FACTOR_PRED */ "homTauDecFacPredictor",
   /* FLAG_HOMOTOPY_TAU_INC_FACTOR */      "homTauIncFac",
@@ -98,6 +102,7 @@ const char *FLAG_NAME[FLAG_MAX+1] = {
   /* FLAG_LSS_MAX_DENSITY */              "lssMaxDensity",
   /* FLAG_LSS_MIN_SIZE */                 "lssMinSize",
   /* FLAG_LV */                           "lv",
+  /* FLAG_LV_TIME */                      "lv_time",
   /* FLAG_MAX_BISECTION_ITERATIONS */     "mbi",
   /* FLAG_MAX_EVENT_ITERATIONS */         "mei",
   /* FLAG_MAX_ORDER */                    "maxIntegrationOrder",
@@ -152,6 +157,9 @@ const char *FLAG_DESC[FLAG_MAX+1] = {
   /* FLAG_CLOCK */                        "selects the type of clock to use -clock=RT, -clock=CYC or -clock=CPU",
   /* FLAG_CPU */                          "dumps the cpu-time into the result file",
   /* FLAG_CSV_OSTEP */                    "value specifies csv-files for debug values for optimizer step",
+  /* FLAG_CVODE_ITER */                   "nonlinear solver iteration for CVODE solver",
+  /* FLAG_CVODE_LMM */                    "linear multistep method for CVODE solver",
+  /* FLAG_DATA_RECONCILE_Cx */            "value specifies a csv-file with inputs as correlation coefficient matrix Cx for DataReconciliation",
   /* FLAG_DAE_MODE */                     "flag to let the integrator use daeResiduals",
   /* FLAG_DELTA_X_LINEARIZE */            "value specifies the delta x value for numerical differentiation used by linearization. The default value is 1e-5.",
   /* FLAG_DELTA_X_SOLVER */               "value specifies the delta x value for numerical differentiation used by integrator. The default values is sqrt(DBL_EPSILON).",
@@ -170,6 +178,7 @@ const char *FLAG_DESC[FLAG_MAX+1] = {
   /* FLAG_HOMOTOPY_MAX_TRIES */           "[int (default 10)] maximum number of tries for one homotopy lambda step",
   /* FLAG_HOMOTOPY_NEG_START_DIR */       "start to run along the homotopy path in the negative direction",
   /* FLAG_HOMOTOPY_ON_FIRST_TRY */        "directly use the homotopy method to solve the initialization problem",
+  /* FLAG_NO_HOMOTOPY_ON_FIRST_TRY */     "disable the use of the homotopy method to solve the initialization problem",
   /* FLAG_HOMOTOPY_TAU_DEC_FACTOR */      "[double (default 10.0)] decrease homotopy step size tau by this factor if tau is too big in the homotopy corrector step",
   /* FLAG_HOMOTOPY_TAU_DEC_FACTOR_PRED */ "[double (default 2.0)] decrease homotopy step size tau by this factor if tau is too big in the homotopy predictor step",
   /* FLAG_HOMOTOPY_TAU_INC_FACTOR */      "[double (default 2.0)] increase homotopy step size tau by this factor if tau is too small in the homotopy corrector step",
@@ -185,10 +194,10 @@ const char *FLAG_DESC[FLAG_MAX+1] = {
   /* FLAG_IDA_SCALING */                  "enable scaling of the IDA solver",
   /* FLAG_IDAS */                         "flag to add sensitivity information to the result files",
   /* FLAG_IGNORE_HIDERESULT */            "ignore HideResult=true annotation",
-  /* FLAG_IIF */                          "value specifies an external file for the initialization of the model",
+  /* FLAG_IIF */                          "value specifies an external file for the initialization of the model relative to -inputPath",
   /* FLAG_IIM */                          "value specifies the initialization method",
   /* FLAG_IIT */                          "[double] value specifies a time for the initialization of the model",
-  /* FLAG_ILS */                          "[int] default: 4",
+  /* FLAG_ILS */                          "[int (default 3)] number of lambda steps for homotopy methods",
   /* FLAG_IMPRK_ORDER */                  "[int (default 5)] value specifies the integration order of the implicit Runge-Kutta method. Valid values: 1-6",
   /* FLAG_IMPRK_LS */                     "selects the linear solver of the integration methods: impeuler, trapezoid and imprungekuta",
   /* FLAG_INITIAL_STEP_SIZE */            "value specifies an initial step size for supported solver",
@@ -210,8 +219,9 @@ const char *FLAG_DESC[FLAG_MAX+1] = {
   /* FLAG_LS_IPOPT */                     "value specifies the linear solver method for ipopt",
   /* FLAG_LSS */                          "value specifies the linear sparse solver method (default: umfpack)",
   /* FLAG_LSS_MAX_DENSITY */              "[double (default 0.2)] value specifies the maximum density for using a linear sparse solver",
-  /* FLAG_LSS_MIN_SIZE */                 "[int (default 4001)] value specifies the minimum system size for using a linear sparse solver",
+  /* FLAG_LSS_MIN_SIZE */                 "[int (default " EXPANDSTRING(DEFAULT_FLAG_LSS_MIN_SIZE) ")] value specifies the minimum system size for using a linear sparse solver",
   /* FLAG_LV */                           "[string list] value specifies the logging level",
+  /* FLAG_LV_TIME */                      "[double list] specifying time interval to allow loging in",
   /* FLAG_MAX_BISECTION_ITERATIONS */     "[int (default 0)] value specifies the maximum number of bisection iterations for state event detection or zero for default behavior",
   /* FLAG_MAX_EVENT_ITERATIONS */         "[int (default 20)] value specifies the maximum number of event iterations",
   /* FLAG_MAX_ORDER */                    "value specifies maximum integration order for supported solver",
@@ -225,7 +235,7 @@ const char *FLAG_DESC[FLAG_MAX+1] = {
   /* FLAG_NLS_INFO */                     "outputs detailed information about solving process of non-linear systems into csv files.",
   /* FLAG_NLS_LS */                       "value specifies the linear solver used by the non-linear solver",
   /* FLAG_NLS_MAX_DENSITY */              "[double (default 0.2)] value specifies the maximum density for using a non-linear sparse solver",
-  /* FLAG_NLS_MIN_SIZE */                 "[int (default 10001)] value specifies the minimum system size for using a non-linear sparse solver",
+  /* FLAG_NLS_MIN_SIZE */                 "[int (default " EXPANDSTRING(DEFAULT_FLAG_NLS_MIN_SIZE) ")] value specifies the minimum system size for using a non-linear sparse solver",
   /* FLAG_NOEMIT */                       "do not emit any results to the result file",
   /* FLAG_NOEQUIDISTANT_GRID */           "stores results not in equidistant time grid as given by stepSize or numberOfIntervals, instead the variable step size of dassl or ida integrator.",
   /* FLAG_NOEQUIDISTANT_OUT_FREQ*/        "value controls the output frequency in noEquidistantTimeGrid mode",
@@ -273,6 +283,20 @@ const char *FLAG_DETAILED_DESC[FLAG_MAX+1] = {
   "  Dumps the cpu-time into the result file using the variable named $cpuTime.",
   /* FLAG_CSV_OSTEP */
   "  Value specifies csv-files for debug values for optimizer step.",
+  /* FLAG_CVODE_ITER */
+  "  Nonlinear solver iteration for CVODE solver. Default: Depends on flag cvodeLinearMultistepMethod. Valid values\n\n"
+  "  * CV_ITER_NEWTON       - Newton iteration.\n"
+  "                           Advised to use together with flag -cvodeLinearMultistepMethod=CV_BDF.\n"
+  "  * CV_ITER_FIXED_POINT  - Fixed-Point iteration iteration.\n"
+  "                           Advised to use together with flag -cvodeLinearMultistepMethod=CV_ADAMS.",
+  /* FLAG_CVODE_LMM */
+  "  Linear multistep method for CVODE solver. Default: CV_BDF. Valid values\n\n"
+  "  * CV_BDF    - BDF linear multistep method for stiff problems.\n"
+  "                Use together with flag -cvodeNonlinearSolverIteration=CV_ITER_NEWTON or don't set cvodeNonlinearSolverIteration.\n"
+  "  * CV_ADAMS  - Adams-Moulton linear multistep method for nonstiff problems.\n"
+  "                Use together with flag -cvodeNonlinearSolverIteration=CV_ITER_FIXED_POINT or don't set cvodeNonlinearSolverIteration.",
+  /* FLAG_DATA_RECONCILE_Cx */
+  "  Value specifies an csv-file with inputs as correlation coefficient matrix Cx for DataReconciliation",
   /* FLAG_DAE_MODE */
   "  Enables daeMode simulation if the model was compiled with the omc flag --daeMode and ida method is used.",
   /* FLAG_DELTA_X_LINEARIZE */
@@ -320,6 +344,9 @@ const char *FLAG_DETAILED_DESC[FLAG_MAX+1] = {
   /* FLAG_HOMOTOPY_ON_FIRST_TRY */
   "  If the model contains the homotopy operator, directly use the homotopy method to solve the initialization problem.\n"
   "  Without this flag, the solver first tries to solve the initialization problem without homotopy and only uses homotopy as fallback option.",
+  /* FLAG_NO_HOMOTOPY_ON_FIRST_TRY */
+  "  Disable the use of the homotopy method to solve the initialization problem.\n"
+  "  Without this flag, the solver tries to solve the initialization problem with homotopy if the model contains the homotopy-operator.",
   /* FLAG_HOMOTOPY_TAU_DEC_FACTOR */
   "  Decrease homotopy step size tau by this factor if tau is too big in the homotopy corrector step (default: 10.0).",
   /* FLAG_HOMOTOPY_TAU_DEC_FACTOR_PRED */
@@ -358,7 +385,7 @@ const char *FLAG_DETAILED_DESC[FLAG_MAX+1] = {
   "  Value [Real] specifies a time for the initialization of the model.",
   /* FLAG_ILS */
   "  Value specifies the number of steps for homotopy method (required: -iim=symbolic).\n"
-  "  The value is an Integer with default value 4.",
+  "  The value is an Integer with default value 3.",
   /* FLAG_IMPRK_ORDER */
   "  Value specifies the integration order of the implicit Runge-Kutta method. Valid values: 1 to 6. Default order is 5.",
   /* FLAG_IMPRK_LS */
@@ -415,6 +442,10 @@ const char *FLAG_DETAILED_DESC[FLAG_MAX+1] = {
   /* FLAG_LV */
   "  Value (a comma-separated String list) specifies which logging levels to\n"
   "  enable. Multiple options can be enabled at the same time.",
+  /* FLAG_LV_TIME */
+  "  Interval (a comma-separated Double list with two elements) specifies in which\n"
+  "  time interval logging is active. Doesn't affect LOG_STDOUT, LOG_ASSERT, and\n"
+  "  LOG_SUCCESS, LOG_STATS, LOG_STATS_V.",
   /* FLAG_MAX_BISECTION_ITERATIONS */
   "  Value specifies the maximum number of bisection iterations for state event\n"
   "  detection or zero for default behavior",
@@ -541,6 +572,9 @@ const int FLAG_TYPE[FLAG_MAX] = {
   /* FLAG_CLOCK */                        FLAG_TYPE_OPTION,
   /* FLAG_CPU */                          FLAG_TYPE_FLAG,
   /* FLAG_CSV_OSTEP */                    FLAG_TYPE_OPTION,
+  /* FLAG_CVODE_ITER */                   FLAG_TYPE_OPTION,
+  /* FLAG_CVODE_LMM */                    FLAG_TYPE_OPTION,
+  /* FLAG_DATA_RECONCILE_Cx */            FLAG_TYPE_OPTION,
   /* FLAG_DAE_SOLVING */                  FLAG_TYPE_FLAG,
   /* FLAG_DELTA_X_LINEARIZE */            FLAG_TYPE_OPTION,
   /* FLAG_DELTA_X_SOLVER */               FLAG_TYPE_OPTION,
@@ -559,6 +593,7 @@ const int FLAG_TYPE[FLAG_MAX] = {
   /* FLAG_HOMOTOPY_MAX_TRIES */           FLAG_TYPE_OPTION,
   /* FLAG_HOMOTOPY_NEG_START_DIR */       FLAG_TYPE_FLAG,
   /* FLAG_HOMOTOPY_ON_FIRST_TRY */        FLAG_TYPE_FLAG,
+  /* FLAG_NO_HOMOTOPY_ON_FIRST_TRY */     FLAG_TYPE_FLAG,
   /* FLAG_HOMOTOPY_TAU_DEC_FACTOR */      FLAG_TYPE_OPTION,
   /* FLAG_HOMOTOPY_TAU_DEC_FACTOR_PRED */ FLAG_TYPE_OPTION,
   /* FLAG_HOMOTOPY_TAU_INC_FACTOR */      FLAG_TYPE_OPTION,
@@ -601,6 +636,7 @@ const int FLAG_TYPE[FLAG_MAX] = {
   /* FLAG_LSS_MAX_DENSITY */              FLAG_TYPE_OPTION,
   /* FLAG_LSS_MIN_SIZE */                 FLAG_TYPE_OPTION,
   /* FLAG_LV */                           FLAG_TYPE_OPTION,
+  /* FLAG_LV_TIME */                      FLAG_TYPE_OPTION,
   /* FLAG_MAX_BISECTION_ITERATIONS */     FLAG_TYPE_OPTION,
   /* FLAG_MAX_EVENT_ITERATIONS */         FLAG_TYPE_OPTION,
   /* FLAG_MAX_ORDER */                    FLAG_TYPE_OPTION,
@@ -656,6 +692,7 @@ const char *SOLVER_METHOD_NAME[S_MAX] = {
   /* S_IRKSCO */        "irksco",
   /* S_DASSL */         "dassl",
   /* S_IDA */           "ida",
+  /* S_CVODE */         "cvode",
   /* S_ERKSSC */        "rungekuttaSsc",
   /* S_SYM_SOLVER */    "symSolver",
   /* S_SYM_SOLVER_SSC */"symSolverSsc",
@@ -674,6 +711,7 @@ const char *SOLVER_METHOD_DESC[S_MAX] = {
   /* S_IRKSCO */        "irksco - own developed Runge-Kutta solver - implicit, step size control, order 1-2",
   /* S_DASSL */         "dassl - default solver - BDF method - implicit, step size control, order 1-5",
   /* S_IDA */           "ida - SUNDIALS IDA solver - BDF method with sparse linear solver - implicit, step size control, order 1-5",
+  /* S_CVODE */         "cvode - experimental implementation of SUNDIALS CVODE solver - BDF or Adams-Moulton method - step size control, order 1-12",
   /* S_ERKSSC */        "rungekuttaSsc - Runge-Kutta based on Novikov (2016) - explicit, step size control, order 4-5 [experimental]",
   /* S_SYM_SOLVER */     "symSolver - symbolic inline Solver [compiler flag +symSolver needed] - fixed step size, order 1",
   /* S_SYM_SOLVER_SSC */ "symSolverSsc - symbolic implicit Euler with step size control [compiler flag +symSolver needed] - step size control, order 1",
@@ -887,3 +925,9 @@ const char *HOM_BACK_STRAT_DESC[HOM_BACK_STRAT_MAX] = {
   /* HOM_BACK_STRAT_FIX */          "go back to the path by fixing one coordinate",
   /* HOM_BACK_STRAT_ORTHOGONAL */   "go back to the path in an orthogonal direction to the tangent vector"
 };
+
+const int FMU_FLAG_MAP[FMU_FLAG_MAX] = {
+ /* FMU_FLAG_UNKNOWN */             FLAG_UNKNOWN,
+ /* FMU_FLAG_SOLVER */              FLAG_S,
+ /* FMU_FLAG_NLS */                 FLAG_NLS
+};
