@@ -30,10 +30,11 @@
 
 
 #include "string_array.h"
-#include "gc/omc_gc.h"
+#include "../gc/omc_gc.h"
 #include "index_spec.h"
 #include "modelica_string.h"
 #include "omc_error.h"
+#include "generic_array.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -89,21 +90,6 @@ void alloc_string_array_data(string_array_t* a)
     a->data = string_alloc(base_array_nr_of_elements(*a));
 }
 
-void copy_string_array_data(const string_array_t source, string_array_t *dest)
-{
-    size_t i, nr_of_elements;
-
-    assert(base_array_ok(&source));
-    assert(base_array_ok(dest));
-    assert(base_array_shape_eq(&source, dest));
-
-    nr_of_elements = base_array_nr_of_elements(source);
-
-    for(i = 0; i < nr_of_elements; ++i) {
-        string_set(dest, i, string_get(source, i));
-    }
-}
-
 void copy_string_array_data_mem(const string_array_t source, modelica_string *dest)
 {
     size_t i, nr_of_elements;
@@ -119,9 +105,7 @@ void copy_string_array_data_mem(const string_array_t source, modelica_string *de
 
 void copy_string_array(const string_array_t source, string_array_t *dest)
 {
-    clone_base_array_spec(&source, dest);
-    alloc_string_array_data(dest);
-    copy_string_array_data(source,dest);
+    string_array_alloc_copy(source,*dest);
 }
 
 /*
@@ -482,31 +466,6 @@ void array_alloc_scalar_string_array(string_array_t* dest, int n,
     va_end(ap);
 }
 
-modelica_string* string_array_element_addr1(const string_array_t * source,int ndims,
-                                              int dim1)
-{
-    return string_ptrget(source, dim1 - 1);
-}
-
-modelica_string* string_array_element_addr2(const string_array_t * source,int ndims,
-                                              int dim1,int dim2)
-{
-    return string_ptrget(source, ((dim1 - 1) * source->dim_size[1]) + (dim2 - 1));
-}
-
-modelica_string* string_array_element_addr(const string_array_t * source,
-                                             int ndims,...)
-{
-    va_list ap;
-    modelica_string* tmp;
-
-    va_start(ap,ndims);
-    tmp = string_ptrget(source, calc_base_index_va(source, ndims, ap));
-    va_end(ap);
-
-    return tmp;
-}
-
 
 /* function: cat_string_array
  *
@@ -796,7 +755,7 @@ void transpose_string_array(const string_array_t * a, string_array_t* dest)
     size_t n,m;
 
     if(a->ndims == 1) {
-        copy_string_array_data(*a,dest);
+        string_array_copy_data(*a, *dest);
         return;
     }
 
