@@ -49,6 +49,11 @@ void create_array_from_shape(const spec_type& sp,BaseArray<T>& s,BaseArray<T>& d
 template < typename T >
 void fill_array_from_shape(const spec_type& sp,BaseArray<T>& s,BaseArray<T>& d);
 
+void BOOST_EXTENSION_EXPORT_DECL identity_alloc(size_t n, DynArrayDim2<int>& I);
+
+template <typename T>
+void diagonal_alloc(const BaseArray<T>& v, BaseArray<T>& D);
+
 template <typename T>
 void promote_array(size_t n, const BaseArray<T>& s, BaseArray<T>& d);
 
@@ -64,6 +69,12 @@ void multiply_array_elem_wise(const BaseArray<T> &leftArray, const BaseArray<T>
 template <typename T>
 void divide_array(const BaseArray<T>& inputArray, const T &b, BaseArray<T>& outputArray);
 
+template <typename T>
+void divide_array(const T &b, const BaseArray<T>& inputArray, BaseArray<T>& outputArray);
+
+template <typename T>
+void divide_array_elem_wise(const BaseArray<T> &leftArray, const BaseArray<T> &rightArray, BaseArray<T> &resultArray);
+
 template <typename T>
 void fill_array(BaseArray<T>& inputArray, T b);
 
@@ -82,12 +93,18 @@ void subtract_array_scalar(const BaseArray<T>& inputArray, T b, BaseArray<T>& ou
 template <typename T>
 void add_array(const BaseArray<T>& leftArray, const BaseArray<T>& rightArray, BaseArray<T>& resultArray);
 
+template <typename T>
+void add_array_scalar(const BaseArray<T>& inputArray, T b, BaseArray<T>& outputArray);
+
 template <typename T>
 void usub_array(const BaseArray<T>& a , BaseArray<T>& b);
 
 template < typename T >
 T sum_array(const BaseArray<T>& x);
 
+template < typename T >
+T product_array(const BaseArray<T>& x);
+
 /**
 finds min/max elements of an array */
 template <typename T>
