@@ -44,8 +44,8 @@ extern "C" {
 }
 
 #include <QtGlobal>
-#if (QT_VERSION < QT_VERSION_CHECK(4, 6, 0))
-#error "OMEdit requires Qt 4.6.0 or newer"
+#if (QT_VERSION < QT_VERSION_CHECK(5, 0, 0))
+#error "OMEdit requires Qt 5.0.0 or newer"
 #endif
 
 #include <QMainWindow>
@@ -159,6 +159,7 @@ public:
   QAction* getResetZoomAction() {return mpResetZoomAction;}
   QAction* getZoomInAction() {return mpZoomInAction;}
   QAction* getZoomOutAction() {return mpZoomOutAction;}
+  QAction* getFitToDiagramAction() {return mpFitToDiagramAction;}
   QAction* getCloseAllWindowsAction() {return mpCloseAllWindowsAction;}
   QAction* getCloseAllWindowsButThisAction() {return mpCloseAllWindowsButThisAction;}
   QAction* getSimulationSetupAction() {return mpSimulationSetupAction;}
@@ -168,6 +169,7 @@ public:
 #if !defined(WITHOUT_OSG)
   QAction* getSimulateWithAnimationAction() {return mpSimulateWithAnimationAction;}
 #endif
+  QAction* getSimulateModelInteractiveAction() {return mpSimulateModelInteractiveAction;}
   QAction* getCheckModelAction() {return mpCheckModelAction;}
   QAction* getCheckAllModelsAction() {return mpCheckAllModelsAction;}
   QAction* getInstantiateModelAction() {return mpInstantiateModelAction;}
@@ -208,6 +210,7 @@ public:
   QAction* getCommitFilesAction() {return mpCommitFilesAction;}
   QAction* getRevertCommitAction() {return mpRevertCommitAction;}
   QAction* getCleanWorkingDirectoryAction() {return mpCleanWorkingDirectoryAction;}
+  QMenu* getNewModelMenu() const {return mpNewModelMenu;}
   QToolBar* getShapesToolBar() const {return mpShapesToolBar;}
   QToolBar* getCheckToolBar() const {return mpCheckToolBar;}
   QToolBar* getSimulationToolBar() const {return mpSimulationToolBar;}
@@ -220,7 +223,7 @@ public:
   int askForExit();
   void beforeClosingMainWindow();
   void openDroppedFile(const QMimeData *pMimeData);
-  void openResultFiles(QStringList fileNames);
+  void openResultFile(const QString &fileName);
   void simulate(LibraryTreeItem *pLibraryTreeItem);
   void simulateWithTransformationalDebugger(LibraryTreeItem *pLibraryTreeItem);
   void simulateWithAlgorithmicDebugger(LibraryTreeItem *pLibraryTreeItem);
@@ -245,12 +248,9 @@ public:
   TransformationsWidget* showTransformationsWidget(QString fileName);
   void findFileAndGoToLine(QString fileName, QString lineNumber);
   void printStandardOutAndErrorFilesMessages();
-  static void PlotCallbackFunction(void *p, int externalWindow, const char* filename, const char* title, const char* grid,
-                                   const char* plotType, const char* logX, const char* logY, const char* xLabel, const char* yLabel,
-                                   const char* x1, const char* x2, const char* y1, const char* y2, const char* curveWidth,
-                                   const char* curveStyle, const char* legendPosition, const char* footer, const char* autoScale,
-                                   const char* variables);
-  static void OMSSimulationFinished(const QString &resultFilePath, QDateTime resultFileLastModifiedDateTime);
+  static void PlotCallbackFunction(void *p, int externalWindow, const char* filename, const char* title, const char* grid, const char* plotType, const char* logX,
+                                   const char* logY, const char* xLabel, const char* yLabel, const char* x1, const char* x2, const char* y1, const char* y2, const char* curveWidth,
+                                   const char* curveStyle, const char* legendPosition, const char* footer, const char* autoScale, const char* variables);
 
   QList<QString> mFMUDirectoriesList;
   QList<QString> mMOLDirectoriesList;
@@ -305,6 +305,7 @@ private:
   // File Menu
   // Modelica File Actions
   QAction *mpNewModelicaClassAction;
+  QAction *mpNewSSPModelAction;
   QAction *mpOpenModelicaFileAction;
   QAction *mpOpenModelicaFileWithEncodingAction;
   QAction *mpLoadModelicaLibraryAction;
@@ -315,9 +316,6 @@ private:
   QAction *mpNewCompositeModelFileAction;
   QAction *mpOpenCompositeModelFileAction;
   QAction *mpLoadExternModelAction;
-  // OMSimulator File Actions
-  QAction *mpNewOMSimulatorModelAction;
-  QAction *mpOpenOMSModelFileAction;
   QAction *mpOpenDirectoryAction;
   QAction *mpSaveAction;
   QAction *mpSaveAsAction;
@@ -347,6 +345,7 @@ private:
   QAction *mpResetZoomAction;
   QAction *mpZoomInAction;
   QAction *mpZoomOutAction;
+  QAction *mpFitToDiagramAction;
   QAction *mpCloseWindowAction;
   QAction *mpCloseAllWindowsAction;
   QAction *mpCloseAllWindowsButThisAction;
@@ -365,6 +364,8 @@ private:
 #if !defined(WITHOUT_OSG)
   QAction *mpSimulateWithAnimationAction;
 #endif
+  QAction *mpSimulateModelInteractiveAction;
+  QAction *mpArchivedSimulationsAction;
   // Debug Menu
   QAction *mpDebugConfigurationsAction;
   QAction *mpAttachDebuggerToRunningProcessAction;
@@ -439,6 +440,7 @@ private:
   QAction *mpAddSubModelAction;
   QAction *mpOMSSimulateAction;
   // Toolbars
+  QMenu *mpNewModelMenu;
   QMenu *mpRecentFilesMenu;
   QMenu *mpLibrariesMenu;
   QToolBar *mpFileToolBar;
@@ -466,6 +468,7 @@ public slots:
   void switchToAlgorithmicDebuggingPerspectiveSlot();
   void showSearchBrowser();
   void createNewModelicaClass();
+  void createNewSSPModel();
   void openModelicaFile();
   void showOpenModelicaFileDialog();
   void loadModelicaLibrary();
@@ -475,8 +478,6 @@ public slots:
   void createNewCompositeModelFile();
   void openCompositeModelFile();
   void loadExternalModels();
-  void createNewOMSModel();
-  void openOMSModelFile();
   void openDirectory();
   void loadSystemLibrary();
   void writeOutputFileData(QString data);
@@ -504,7 +505,9 @@ public slots:
   void simulateModel();
   void simulateModelWithTransformationalDebugger();
   void simulateModelWithAlgorithmicDebugger();
+  void showArchivedSimulations();
   void simulateModelWithAnimation();
+  void simulateModelInteractive();
   void exportModelFMU();
   void importModelFMU();
   void importFMUModelDescription();
@@ -598,12 +601,17 @@ private:
   QRadioButton *mpMSL3RadioButton;
   QRadioButton *mpMSL4RadioButton;
   QRadioButton *mpNoMSLRadioButton;
+  QWidget *mpWidget;
 private slots:
   void setMSLVersion();
 
   // QDialog interface
 public slots:
   virtual void reject() override;
+
+  // QWidget interface
+public:
+  virtual QSize sizeHint() const override;
 };
 
 #endif // MAINWINDOW_H
