@@ -1583,11 +1583,14 @@ void RenameCompositeModelCommand::undo()
  * \param oldSnapshot
  * \param newSnapshot
  * \param editedCref - Cref where the change has happened.
+ * \param doSnapShot
+ * \param switchToEdited
  * \param commandText
  * \param pParent
  */
 OMSimulatorUndoCommand::OMSimulatorUndoCommand(const QString &modelName, const QString &oldSnapshot, const QString &newSnapshot, const QString &editedCref,
-                                               const bool doSnapShot, const QString &commandText, UndoCommand *pParent)
+                                               const bool doSnapShot, const bool switchToEdited, const QString oldEditedCref, const QString newEditedCref,
+                                               const QString &commandText, UndoCommand *pParent)
   : UndoCommand(pParent)
 {
   mModelName = modelName;
@@ -1595,6 +1598,9 @@ OMSimulatorUndoCommand::OMSimulatorUndoCommand(const QString &modelName, const Q
   mNewSnapshot = newSnapshot;
   mEditedCref = editedCref;
   mDoSnapShot = doSnapShot;
+  mSwitchToEdited = switchToEdited;
+  mOldEditedCref = oldEditedCref;
+  mNewEditedCref = newEditedCref;
   mExpandedLibraryTreeItemsList.clear();
   mOpenedModelWidgetsList.clear();
   mIconSelectedItemsList.clear();
@@ -1620,10 +1626,10 @@ void OMSimulatorUndoCommand::redoInternal()
   MainWindow::instance()->getModelWidgetContainer()->getOpenedModelWidgetsOfOMSimulatorModel(mModelName, &mOpenedModelWidgetsList);
   // load the new snapshot
   if (mDoSnapShot) {
-    OMSProxy::instance()->loadSnapshot(mModelName, mNewSnapshot);
+    OMSProxy::instance()->importSnapshot(mModelName, mNewSnapshot, &mModelName);
   }
   // reload/redraw the OMSimulator model
-  MainWindow::instance()->getLibraryWidget()->getLibraryTreeModel()->reLoadOMSimulatorModel(mModelName, mEditedCref, mNewSnapshot);
+  MainWindow::instance()->getLibraryWidget()->getLibraryTreeModel()->reLoadOMSimulatorModel(mModelName, mEditedCref, mNewSnapshot, mOldEditedCref, mNewEditedCref);
   // Get the new model LibraryTreeItem
   LibraryTreeItem *pNewModelLibraryTreeItem = pLibraryTreeModel->findLibraryTreeItemOneLevel(mModelName);
   assert(pNewModelLibraryTreeItem);
@@ -1646,9 +1652,9 @@ void OMSimulatorUndoCommand::undo()
 {
   // load the old snapshot
   if (mDoSnapShot) {
-    OMSProxy::instance()->loadSnapshot(mModelName, mOldSnapshot);
+    OMSProxy::instance()->importSnapshot(mModelName, mOldSnapshot, &mModelName);
   }
-  MainWindow::instance()->getLibraryWidget()->getLibraryTreeModel()->reLoadOMSimulatorModel(mModelName, mEditedCref, mOldSnapshot);
+  MainWindow::instance()->getLibraryWidget()->getLibraryTreeModel()->reLoadOMSimulatorModel(mModelName, mEditedCref, mOldSnapshot, mNewEditedCref, mOldEditedCref);
   // Get the new model LibraryTreeItem
   LibraryTreeModel *pLibraryTreeModel = MainWindow::instance()->getLibraryWidget()->getLibraryTreeModel();
   LibraryTreeItem *pNewModelLibraryTreeItem = pLibraryTreeModel->findLibraryTreeItemOneLevel(mModelName);
@@ -1682,35 +1688,11 @@ void OMSimulatorUndoCommand::restoreClosedModelWidgets()
  */
 void OMSimulatorUndoCommand::switchToEditedModelWidget()
 {
-  LibraryTreeModel *pLibraryTreeModel = MainWindow::instance()->getLibraryWidget()->getLibraryTreeModel();
-  LibraryTreeItem *pEditedLibraryTreeItem = pLibraryTreeModel->findLibraryTreeItem(mEditedCref);
-  assert(pEditedLibraryTreeItem);
-  pLibraryTreeModel->showModelWidget(pEditedLibraryTreeItem);
-}
-
-OMSRenameCommand::OMSRenameCommand(LibraryTreeItem *pLibraryTreeItem, QString name, UndoCommand *pParent)
-  : UndoCommand(pParent)
-{
-  mpLibraryTreeItem = pLibraryTreeItem;
-  mOldName = mpLibraryTreeItem->getName();
-  mNewName = name;
-  setText("OMS rename");
-}
-
-/*!
- * \brief OMSRenameCommand::redoInternal
- * redoInternal the OMSRenameCommand
- */
-void OMSRenameCommand::redoInternal()
-{
-  qDebug() << "OMSRenameCommand::redoInternal() not implemented.";
-}
-
-/*
- * \brief OMSRenameCommand::undo
- * Undo the OMSRenameCommand
- */
-void OMSRenameCommand::undo()
-{
-  qDebug() << "OMSRenameCommand::undo() not implemented.";
+  if (mSwitchToEdited) {
+    LibraryTreeModel *pLibraryTreeModel = MainWindow::instance()->getLibraryWidget()->getLibraryTreeModel();
+    LibraryTreeItem *pEditedLibraryTreeItem = pLibraryTreeModel->findLibraryTreeItem(mEditedCref);
+    if (pEditedLibraryTreeItem) {
+      pLibraryTreeModel->showModelWidget(pEditedLibraryTreeItem);
+    }
+  }
 }
