@@ -45,7 +45,7 @@ QString Helper::OpenModelicaVersion = "";
 QString Helper::OpenModelicaHome = "";
 QString Helper::OpenModelicaLibrary = "";
 QString Helper::OMCServerName = "OMEdit";
-QString Helper::omFileTypes = "All Modelica Files (*.mo *.mol);;Modelica Files (*.mo);;Encrypted Modelica Libraries (*.mol)";
+QString Helper::omFileTypes = "All Files (*.mo *.mol *.ssp);;Modelica Files (*.mo);;Encrypted Modelica Libraries (*.mol);;System Structure and Parameterization Files (*.ssp)";
 QString Helper::omEncryptedFileTypes = "Encrypted Modelica Libraries (*.mol)";
 QString Helper::omnotebookFileTypes = "OMNotebook Files (*.onb *.onbz *.nb)";
 QString Helper::ngspiceNetlistFileTypes = "ngspice Netlist Files (*.cir *.sp *.spice)";
@@ -66,7 +66,6 @@ QString Helper::exeFileTypes = "Executable files (*)";
 QString Helper::txtFileTypes = "TXT Files (*.txt)";
 QString Helper::figaroFileTypes = "Figaro Files (*.fi)";
 QString Helper::visualizationFileTypes = "Visualization Files (*.mat *.csv *.fmu);;Visualization MAT(*.mat);;Visualization CSV(*.csv);;Visualization FMU(*.fmu)";
-QString Helper::omsFileTypes = "Composite Model Files (*.ssp)";
 QString Helper::subModelFileTypes = "SubModel Files (*.fmu *.mat *.csv);;SubModel FMU (*.fmu);;SubModel MAT (*.mat);;SubModel CSV (*.csv)";
 int Helper::treeIndentation = 13;
 QSize Helper::iconSize = QSize(20, 20);
@@ -145,6 +144,7 @@ QString Helper::close;
 QString Helper::error;
 QString Helper::chooseFile;
 QString Helper::chooseFiles;
+QString Helper::saveFile;
 QString Helper::attributes;
 QString Helper::properties;
 QString Helper::add;
@@ -205,6 +205,7 @@ QString Helper::paste;
 QString Helper::resetZoom;
 QString Helper::zoomIn;
 QString Helper::zoomOut;
+QString Helper::fitToDiagram;
 QString Helper::loading;
 QString Helper::question;
 QString Helper::search;
@@ -441,6 +442,7 @@ void Helper::initHelperVariables()
   Helper::error = tr("Error");
   Helper::chooseFile = tr("Choose File");
   Helper::chooseFiles = tr("Choose File(s)");
+  Helper::saveFile = tr("Save File");
   Helper::attributes = tr("Attributes");
   Helper::properties = tr("Properties");
   Helper::add = tr("Add");
@@ -502,6 +504,7 @@ void Helper::initHelperVariables()
   Helper::resetZoom = tr("Reset Zoom");
   Helper::zoomIn = tr("Zoom In");
   Helper::zoomOut = tr("Zoom Out");
+  Helper::fitToDiagram = tr("Fit to Diagram");
   Helper::loading = tr("Loading");
   Helper::question = tr("Question");
   Helper::search = tr("Search");
