@@ -34,8 +34,9 @@
 
 #include "Component.h"
 #include "ComRef.h"
-#include "Values.h"
 #include "ResultWriter.h"
+#include "Snapshot.h"
+#include "Values.h"
 #include "Variable.h"
 
 #include <fmilib.h>
@@ -53,11 +54,13 @@ namespace oms
     ~ComponentFMUME();
 
     static Component* NewComponent(const oms::ComRef& cref, System* parentSystem, const std::string& fmuPath);
-    static Component* NewComponent(const pugi::xml_node& node, System* parentSystem,  const std::string& sspVersion);
+    static Component* NewComponent(const pugi::xml_node& node, System* parentSystem,  const std::string& sspVersion, const Snapshot& snapshot);
     const FMUInfo* getFMUInfo() const {return &(this->fmuInfo);}
 
-    oms_status_enu_t exportToSSD(pugi::xml_node& node, pugi::xml_node& ssvNode) const;
+    oms_status_enu_t exportToSSD(pugi::xml_node& node, Snapshot& snapshot) const;
+    oms_status_enu_t exportToSSV(pugi::xml_node& ssvNode);
     oms_status_enu_t exportToSSVTemplate(pugi::xml_node& ssvNode);
+    oms_status_enu_t exportToSSMTemplate(pugi::xml_node& ssmNode);
     oms_status_enu_t instantiate();
     oms_status_enu_t initialize();
     oms_status_enu_t terminate();
@@ -102,6 +105,12 @@ namespace oms
 
     oms_status_enu_t setFaultInjection(const ComRef& signal, oms_fault_type_enu_t faultType, double faultValue);
 
+    void getFilteredSignals(std::vector<Connector>& filteredSignals) const;
+
+    oms_status_enu_t addResources(std::string& filename);
+    oms_status_enu_t setResourcesHelper1(Values value);
+    oms_status_enu_t setResourcesHelper2(Values value);
+
   protected:
     ComponentFMUME(const ComRef& cref, System* parentSystem, const std::string& fmuPath);
 
@@ -109,6 +118,8 @@ namespace oms
     ComponentFMUME(ComponentFMUME const& copy);            ///< not implemented
     ComponentFMUME& operator=(ComponentFMUME const& copy); ///< not implemented
 
+    oms_status_enu_t renameValues(const ComRef& oldCref, const ComRef& newCref);
+
   private:
     jm_callbacks callbacks;
     fmi2_callback_functions_t callbackFunctions;
