@@ -34,9 +34,10 @@
 #include "Flags.h"
 #include "System.h"
 #include "Component.h"
+#include "Snapshot.h"
 #include <miniunz.h>
-#include <OMSFileSystem.h>
 #include <time.h>
+#include "ssd/Tags.h"
 
 oms::Scope::Scope()
   : tempDir(".")
@@ -106,19 +107,29 @@ oms_status_enu_t oms::Scope::deleteModel(const oms::ComRef& cref)
 
 oms_status_enu_t oms::Scope::renameModel(const oms::ComRef& cref, const oms::ComRef& newCref)
 {
-  auto it = models_map.find(cref);
+  oms::ComRef tail(cref);
+  oms::ComRef front = tail.pop_front();
+
+  auto it = models_map.find(front);
   if (it == models_map.end())
-    return logError("Model \"" + std::string(cref) + "\" does not exist in the scope");
+    return logError("Model \"" + std::string(front) + "\" does not exist in the scope");
 
   unsigned int index = it->second;
-  oms_status_enu_t status = models[index]->rename(newCref);
-  if (oms_status_ok != status)
-    return status;
+  if (tail.isEmpty()) // renaming the model itself
+  {
+    oms_status_enu_t status = models[index]->rename(newCref);
+    if (oms_status_ok != status)
+      return status;
 
-  models_map.erase(it);
-  models_map[newCref] = index;
+    models_map.erase(it);
+    models_map[newCref] = index;
 
-  return oms_status_ok;
+    return oms_status_ok;
+  }
+  else // renaming a subcomponent
+  {
+    return models[index]->rename(tail, newCref);
+  }
 }
 
 oms_status_enu_t oms::Scope::exportModel(const oms::ComRef& cref, const std::string& filename)
@@ -174,7 +185,8 @@ oms_status_enu_t oms::Scope::miniunz(const std::string& filename, const std::str
 
 oms_status_enu_t oms::Scope::importModel(const std::string& filename, char** _cref)
 {
-  *_cref = NULL;
+  if (_cref)
+    *_cref = NULL;
 
   std::string extension = "";
   if (filename.length() > 4)
@@ -188,12 +200,11 @@ oms_status_enu_t oms::Scope::importModel(const std::string& filename, char** _cr
   if (oms_status_ok != oms::Scope::miniunz(filename, temp_root.string(), true))
     return logError("failed to extract \"SystemStructure.ssd\" from \"" + filename + "\"");
 
-  pugi::xml_document doc;
-  pugi::xml_parse_result result = doc.load_file((temp_root / "SystemStructure.ssd").string().c_str());
-  if (!result)
-    return logError("loading \"" + std::string(filename) + "\" failed (" + std::string(result.description()) + ")");
-
-  const pugi::xml_node node = doc.document_element(); // ssd:SystemStructureDescription
+  Snapshot snapshot;
+  snapshot.importResourceFile("SystemStructure.ssd", temp_root);
+  const pugi::xml_node node = snapshot.getResourceNode("SystemStructure.ssd");
+  if (!node)
+    return logError("failed to load \"SystemStructure.ssd\"");
 
   ComRef cref = ComRef(node.attribute("name").as_string());
   std::string ssdVersion = node.attribute("version").as_string();
@@ -213,7 +224,16 @@ oms_status_enu_t oms::Scope::importModel(const std::string& filename, char** _cr
 
   bool old_copyResources = model->copyResources();
   model->copyResources(false);
-  oms_status_enu_t status = model->importFromSSD(node);
+
+  // add the remaining resources (e.g) .ssv, .ssm and signalFilter.xml to oms:snapshot
+  for (const auto &entry : OMS_RECURSIVE_DIRECTORY_ITERATOR(model->getTempDirectory()))
+    if (entry.path().has_extension())
+      if (".ssv" == entry.path().extension() || ".ssm" == entry.path().extension() || ".xml" == entry.path().extension())
+        snapshot.importResourceFile(naive_uncomplete(entry.path(), model->getTempDirectory()), model->getTempDirectory());
+
+  // snapshot.debugPrintAll();
+
+  oms_status_enu_t status = model->importFromSnapshot(snapshot);
   model->copyResources(old_copyResources);
 
   Scope::GetInstance().setWorkingDirectory(cd);
@@ -375,3 +395,51 @@ oms::Model* oms::Scope::getModel(const oms::ComRef& cref)
 
   return models[it->second];
 }
+
+oms_status_enu_t oms::Scope::loadSnapshot(const oms::ComRef& cref, const char* snapshot, char** newCref)
+{
+  if (newCref)
+    *newCref = NULL;
+
+  pugi::xml_document doc;
+  pugi::xml_parse_result result = doc.load(snapshot);
+  if (!result)
+    return logError("loading snapshot failed (" + std::string(result.description()) + ")");
+
+  const pugi::xml_node node = doc.document_element(); // ssd:SystemStructureDescription
+
+  ComRef name(node.attribute("name").as_string());
+  if (name != cref && getModel(name))
+    return logError("failed to load snapshot, because it would change the model's name but it already exists in the scope");
+
+  std::string ssdVersion = node.attribute("version").as_string();
+  if (ssdVersion != "Draft20180219" && ssdVersion != "1.0")
+    return logError("Unknown SSD version \"" + ssdVersion + "\"; supported version are \"1.0\" and \"Draft20180219\".");
+
+  oms_status_enu_t status = getModel(cref)->loadSnapshot(node);
+
+  ComRef new_name(cref);
+  if (oms_status_ok == status && name != cref)
+    if (oms_status_ok == renameModel(cref, name))
+      new_name = name;
+
+  if (newCref)
+    *newCref = (char*)getModel(new_name)->getCref().c_str();
+
+  return status;
+}
+
+// TODO: renaming not yet supported
+oms_status_enu_t oms::Scope::importSnapshot(const oms::ComRef& cref, const char* snapshot, char** newCref)
+{
+  if (newCref)
+    *newCref = NULL;
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(cref);
+  if (!model)
+    return logError_ModelNotInScope(cref);
+
+  oms_status_enu_t status = model->importSnapshot(snapshot, newCref);
+
+  return status;
+}
